% Solving Ax=e_1 gives no convergence till last step 
% 
%     [0  1          ]
%     [   0  1       ]
% A = [      .  .    ]
%     [         0  1 ] 
%     [a1 a2  ... an ]
%
function GMRES_Example01

clear all;
close all;

n = 64;
A = diag(ones(n-1,1),1);
A(n,:) = 1:n;
b = zeros(n,1); b(1) = 1;

%%%%% 
x = zeros(n,1); % ֵ
tol = 1e-8;     % 
maxit = n;      % 
m = n;          % 

%%%% GMRES ============================================================
flag = 0;
iter = 0;    
relres = 0.0;  

% Initialize workspace
e1 = zeros(n,1);  e1(1) = 1.0;
cs = zeros(m,1);  sn = zeros(m,1);

if (m < maxit) % initialize workspace for restarted GMRES
    H(1:m+1,1:m) = zeros(m+1,m);
    V(1:n,1:m+1) = zeros(n,m+1);
end

bnrm2 = norm(b);
if  (bnrm2 == 0.0), bnrm2 = 1.0; end

r = b - A*x;
res = norm(r);  % residual 
bnrm2 = res;
relres = res / bnrm2;  % relative residual
if (relres < tol),  return, end
% fprintf(' iter=%d, relres=%e\n', iter, norm(r));

%%%%% ͼ
hold on
plot(0,relres,'o');
axis([0,maxit+1,0,1.1]);
relres_old=relres;
%%%%% =========================================================
        
% Begin iteration
for Iter = 1 : ceil(maxit/m)   %  
    V(:,1) = r / res;
    s = res * e1;  % s = norm(r)*e1;
    for i = 1 : m  % construct orthonormal bases using MGS
        w = A*V(:,i); 
        for k = 1 : i 
            H(k,i)= w'*V(:,k);
            w = w - H(k,i)*V(:,k);
        end
        H(i+1,i) = norm(w);
        V(:,i+1) = w / H(i+1,i);
        for k = 1 : i-1 % Apply Givens rotation
            temp     =   cs(k)*H(k,i) + sn(k)*H(k+1,i);
            H(k+1,i) = - sn(k)*H(k,i) + cs(k)*H(k+1,i);
            H(k,i)   =   temp;
        end
        % Form i-th rotation matrix
        [cs(i), sn(i)] = rotmat( H(i,i), H(i+1,i) );
        % Approximate residual norm
        temp   =   cs(i)*s(i);
        s(i+1) = - sn(i)*s(i);
        s(i)   =   temp;
        H(i,i) =   cs(i)*H(i,i) + sn(i)*H(i+1,i);
        H(i+1,i) = 0.0;
        relres  = abs(s(i+1)) / bnrm2;  % Check convergence
        % fprintf(' iter=%d, relres=%e\n', i, relres);
        %%%%% ͼ
        plot([(Iter-1)*m+i-1,(Iter-1)*m+i],[relres_old,relres], 'o-');
        pause(0.1)
        relres_old=relres;
        fprintf('iter=%d,relres=%.4e\n',i,relres)
        %%%%% =========================================================
        if ( relres <= tol )
            y = H(1:i,1:i) \ s(1:i);
            x = V(:,1:i) * y; 
              break;
        end
    end
    if ( relres <= tol ), break, end
    y = H(1:m,1:m) \ s(1:m);
    x = V(:,1:m)*y;  
    r = b-A*x;
    res = norm(r);  % check convergence
    relres = res / bnrm2;
    fprintf(' iter=%d, relres=%e\n', i, relres);
    if ( relres <= tol ), break, end;
end
iter = (Iter - 1) * m + i;  
if ( relres > tol ), flag=1; end  % no convergence 

xlabel('iteration number');
ylabel('norm of relative residual');

%%%%% rotmat ==========================================================
function [ c, s ] = rotmat( a, b )
if ( b == 0.0 ),
    c = 1.0;
    s = 0.0;
elseif ( abs(b) > abs(a) ),
    temp = a / b;
    s = 1.0 / sqrt( 1.0 + temp^2 );
    c = temp * s;
else
    temp = b / a;
    c = 1.0 / sqrt( 1.0 + temp^2 );
    s = temp * c;
end
%%%%% END =============================================================



