%
%  GMRES for PDE
%
clear all;
close all;

% initialization
N = 64; n = N*N;
a = 0; b = 1;
hx = (b-a)/(N+1);
hy = hx;
tol = 1e-6;
maxit = N*N;
x0 = zeros(n,1);   % initial guess

% coefficient matrix : A
T = 2*eye(N) - diag(ones(N-1,1),1) - diag(ones(N-1,1),-1);
D = (diag(ones(N-1,1),1) - diag(ones(N-1,1),-1))/2;
A = kron(eye(N),T) + kron(T,eye(N)) ...
    + kron(eye(N),hy*D) + kron(hx*D,eye(N)) ...
    + hx*hy*eye(N*N);
% A*x
Ax = @(x) A*x;

% f(x,y) and right hand side
fxy = @(x,y) (3-y.*(1+y)).*x.*(1-x) + (3-2*x).*y.*(1-y);
[xtmp,ytmp] = meshgrid(linspace(a,b,N+2),linspace(a,b,N+2));
b = hx*hy*fxy(xtmp(2:N+1,2:N+1),ytmp(2:N+1,2:N+1));
b = b(:);

% true solution -- for testing
ut = xtmp.*ytmp.*(1-xtmp).*(1-ytmp);
ut = ut(2:N+1,2:N+1);

% function [x,flag,relres,iter,relvec] = Gmres(A,b,restart,tol,maxit,x0)
t0 = clock;
[x,flag,relres,iter,relvec] = Gmres(A,b,[],tol,maxit,x0);
t1 = etime(clock,t0);
fprintf('GMRES: iter=%d, time=%.2e, relres=%.3e, relerr=%.3e\n', ...
    iter, t1, relres, norm(x-ut(:))/norm(ut(:)));
semilogy(relvec,'o');  
title('GMRES: iteration history of relative residual')

return

%%%%% 与带重启的 GMRES 进行比较
restart = 30;
t0 = clock;
[x,flag,relres,iter,relvec] = Gmres(A,b,restart,tol,maxit,x0);
t1 = etime(clock,t0);
fprintf('GMRES(%d): iter=%d, time=%.2e, relres=%.3e, relerr=%.3e\n', ...
    restart, iter, t1, relres, norm(x-ut(:))/norm(ut(:)));
legend_GMRESm = ['GMRES(',int2str(restart),')'];
hold on, semilogy(relvec,'+'); legend('GMRES', legend_GMRESm); hold off;
tit = ['GMRES vs GMRES(restart) for n=',int2str(N),'^2'];
title(tit)

return

%%%%% 与 FOM 进行比较
t0 = clock;
[x,relres,iter,relvec] = Fom(A,b,tol,maxit,x0);
t1 = etime(clock,t0);
fprintf('FOM: iter=%d, time=%.2e, relres=%.3e, relerr=%.3e\n', ...
    iter, t1, relres, norm(x-ut(:))/norm(ut(:)));
legend_GMRESm = ['GMRES(',int2str(restart),')']; 
hold on, semilogy(relvec,'v'); legend('GMRES', legend_GMRESm,'FOM'); hold off;
tit = ['GMRES vs GMRES(restart) vs FOM for n=',int2str(N),'^2'];
title(tit)

