%
% SOR and SSOR for the 2-D Poisson problem
% discretized by five point finite difference scheme
%
% function Poisson_SOR_SSOR
%
clear all;
close all;
global u0 ut iter_max n hh f norm_ut tol

n = 32;
h = 1/(n+1); % step size

% initial guess
u0 = zeros(n+2,n+2);

% boundary condition
x = 0:h:1;  y = 0:h:1;
u0(1,:) = 0.25 * (y .* y);
u0(n+2,:) = 0.25 * (1 + y .* y);
u0(:,1) = 0.25 * (x .* x);
u0(:,n+2) = 0.25 * (x .* x + 1);

f = -1; % right-hand side f(x,y)

% true solution
[X,Y] = meshgrid(x);
ut = 0.25*(X.*X + Y.*Y);
norm_ut = norm(ut(:));

tol = 1e-6; % stopping criteria
iter_max = 1000; % maximun number of iterations

hh = h*h;  % h^2

% values of omega
Omega = [1:0.02:1.9];

% SOR iteration for different parameters
% fprintf('===== SOR ===== \n')
i=1;
for omega = Omega
    [k, relerr] = SOR(omega);
    % fprintf('omega=%.4f, iter=%d, relerr=%.4e\n', omega, k, relerr);
    iter_SOR(i)=k;
    i=i+1;
end

% fprintf('===== SSOR ===== \n')
i=1;
for omega = Omega
    [k, relerr] = SSOR(omega);
    % fprintf('omega=%.4f, iter=%d, relerr=%.4e\n', omega, k, relerr);
    iter_SSOR(i)=k;
    i=i+1;    
end

plot(Omega,iter_SOR,'o-b',Omega,iter_SSOR,'+-k');
legend('SOR','SSOR');
title([int2str(n),' x ', int2str(n)]);
%axis(gca,[1,1.9,0,400])

%%%%% ==================================================================
function [k, relerr] = SOR(omega)
global u0 ut iter_max n hh f norm_ut tol
u = u0;
for k = 1 : iter_max
    for i = 2:n+1
        for j = 2:n+1
            vij_old = u(i,j);
            u(i,j) = 0.25* (hh*f + u(i+1,j) + u(i-1,j) ...
                + u(i,j+1) + u(i,j-1));
            u(i,j) = (1-omega)*vij_old + omega*u(i,j);
        end
    end
    relerr = norm(u(:) - ut(:)) / norm_ut;
    if relerr < tol, break; end
end
end

function [k, relerr] = SSOR(omega)
global u0 ut iter_max n hh f norm_ut tol
u = u0;
for k = 1 : iter_max
    for i = 2:n+1
        for j = 2:n+1
            vij_old = u(i,j);
            u(i,j) = 0.25* (hh*f + u(i+1,j) + u(i-1,j) ...
                + u(i,j+1) + u(i,j-1));
            u(i,j) = (1-omega)*vij_old + omega*u(i,j);
        end
    end
    for i = n+1:-1:2
        for j = n+1:-1:2
            vij_old = u(i,j);
            u(i,j) = 0.25* (hh*f + u(i+1,j) + u(i-1,j) ...
                + u(i,j+1) + u(i,j-1));
            u(i,j) = (1-omega)*vij_old + omega*u(i,j);
        end
    end

    relerr = norm(u(:) - ut(:)) / norm_ut;
    if relerr < tol, break; end
end
end


